<?
/*
   Author:  Richard Graham
   Licence: The following content is made available under the terms of the Creative Commons Attribution-Share Alike 2.0 Generic licence. Please see COPYING.TXT for more information.
*/

require("helperfuncts.php");

if (strcmp($_REQUEST["submit"],"")==0) {
   getInput();
}
else {
   if (strcmp($_REQUEST["playing"],"")==0){
      recordResults();
   }
   $rank = makeRank();
   
   print "<center>";
   
   displayAbuse($rank);   
   displayRank($rank, 12);
   
   print "<br><a href='".$_SERVER['PHP_SELF']."'>Try again</a>";
   print "</center>";
   
}

function displayAbuse($rank){
   $keys=array_keys($rank);
   $xmldata = $rank[$keys[0]];
   print "<p>Got ".getDetailFromXML($xmldata, "length")." minutes to spare once you are done ".getAbuse()."? Why not download LugRadio episode ".getDetailFromXML($xmldata, "number").", season ".getDetailFromXML($xmldata, "season")." <i><a href='".getDetailFromXML($xmldata, "url")."'>'".getDetailFromXML($xmldata, "title")."'</a></i> from the LugRadio archives? Alternatively, choose one of the episodes from the list below which we have selected to match your LugRadio Top Trumps scores.</p>";
}

function displayRank($rank, $items){
   $rank = array_slice($rank, 0, $items);
   
   print "<table border='0'>\n";
   foreach ($rank as $episodeXML){
      $date = date('d M y', getDateFromXML($episodeXML));
      print "\n<tr> <td><font size='-1'>$date &nbsp</font></td>";
      //print "<td><font size='-1'>(S".getDetailFromXML($episodeXML,"season")." E".getDetailFromXML($episodeXML,"number").")</font></td>";
      print "<td><i><a href='".getDetailFromXML($episodeXML, "url")."'>".getDetailFromXML($episodeXML, "title")."</a></i></td>";
      print "<td><font size='-1'>&nbsp(".getDetailFromXML($episodeXML,"length")." min)</font></td>";
      print "</td></tr>";
      //print "<br>";
   }
   print "</table>\n";
   
}

function getAbuse(){
   $xmlfile = "abuse.xml";
   $xml = new DOMDocument();
   $xml->load($xmlfile);
   $xpath = new DomXPath($xml);
   
   $actions = array();
   
   foreach (biggestScores() as $category){
      $thisCategoryActions = $xpath->query("/abuse/".$category."/action");
      foreach ($thisCategoryActions as $action) {
         $actions[]=$action->nodeValue;
      }
   }
   
   return $actions[ rand(0,count($actions)-1) ];
}

// Returns a 2-column array of DOM nodes (for each episode) and the score each received
// The score represents the difference between selected and given personality types weighted by the age of the show and longevity factor
function makeRank(){

   // freshness multiplier
   $freshness = 0.25; // goes between 0..1
                      // a large number -> harshly discriminate against old episodes
                      // a small number -> little discrimination

   ////
   
   $traits = array("tinkering", "freedom", "community", "underdog", "cheapness");

   $xmlfile = "episodes.xml";
   $xml = new DOMDocument();
   $xml->load($xmlfile);
   
   $xpath = new DomXPath($xml);
   
   // It is usefull to know the oldest and newest shows made
   $dates = $xpath->query("/episodes/episode/details/date");
   $newest = 0.0; $oldest = time();
   foreach ($dates as $date){
      $timestamp = intval($date->nodeValue);
      if ($timestamp > $newest) { $newest = $timestamp; }
      if ($timestamp < $oldest) { $oldest = $timestamp; } 
   }  
   
   $scores = array(); $xmlEntries=array();
   $episodeNodes = $xpath->query("/episodes/episode");
   foreach ($episodeNodes as $episode){
      // This uses a random undocumented feature of php. Probably not the best thing..
      $episodeXML = $episode->C14N();
      
      // Compute the raw score for the episode
      $difference = 0.0;
      foreach ($traits as $trait){
         $difference = $difference + abs(getScoreFromXML($episodeXML, $trait) - getScore($trait));
      }
      //$score = floatval($difference)/(10.0*floatval(count($traits)));
      //$score = (10.0*floatval(count($traits))) / floatval($difference);
      $score = 1.0 - floatval($difference)/(10.0*floatval(count($traits)));
      
      // Calculate the age of the episode as a number 0..1, 0 being when the show started, 1 the present    
      $age = (getDateFromXML($episodeXML) - $oldest) / ($newest - $oldest);
      
      // Give a weighting score for the episode based of age and longevity
      $weight = $age * (getScoreFromXML($episodeXML, "longevity") / 10.0);
      
      // Combine this weight with the original score somehow to get a weighted score
      $weightedScore = (1.0 - $freshness)*$score + $freshness*$weight;
      
      // Apply a small random shuffling to give some variation from run to run
      $weightedScore = round($weightedScore, 2);
      $weightedScore = $weightedScore + rand(0, 100) * 0.0001;
      
      $rank[10000*$weightedScore]=$episodeXML;
      
      //print "</centre> score = ".strval($score)."   weight = ".strval(round($weight,3))."   weighted = ".strval($weightedScore)."  ".getDetailFromXML($episodeXML, "title")."<br><centre>";
      
   }
   
   krsort($rank);
   return $rank;   
}


function recordResults(){
   // open XML
   
   $xmlfile = "stats.xml";
   $original = new DOMDocument(); 
   $original->load($xmlfile);
   
   // increment the total number of visitors
   $visitors = intval( getXMLField("/statistics/total", $original) );
   changeXMLField("/statistics/total", 1 + $visitors, $original);
   
   // record the averages
   $fields = array("tinkering", "freedom", "community", "underdog", "cheapness");
   foreach ($fields as $field){
      $oldvalue = floatval( getXMLField("/statistics/averages/".$field, $original) );
      $selection = getScore($field);
      $newvalue = ($oldvalue*$visitors + $selection) / ($visitors + 1);
      changeXMLField("/statistics/averages/".$field, $newvalue, $original);   
   }
   
   // find and update the biggest score(s)
   $big = biggestScores();
   $contribution = 1.0/floatval(count($big));
   foreach ($big as $field) {
      $oldvalue = floatval( getXMLField("/statistics/mostImportant/".$field, $original) );
      changeXMLField("/statistics/mostImportant/".$field, $oldvalue + $contribution, $original);
   }
   
   // find and update the smallest score(s)
   $small = smallestScores();
   $contribution = 1.0/floatval(count($small));
   foreach ($small as $field) {
      $oldvalue = floatval( getXMLField("/statistics/leastImportant/".$field, $original) );
      changeXMLField("/statistics/leastImportant/".$field, $oldvalue + $contribution, $original);
   }
   
   $original->save($xmlfile);
   
}


function getInput(){
   
   print "<center><form method='post' name='scores'>";
   
   print "<p>What are your motivations for using free software? Let us know and you will get a personalised list of episodes from the LugRadio archives to listen to!</p>";
   
   print "<p>Are you a crusader for <b>freedom</b>, out there to spread the free software message? Do you like to <b>tinker</b> and play with software ... press buttons, play with grep switches and understand how things work? Are you someone who simply likes to support the <b>underdog</b>? Do you like being part of a <b>community</b> and communitising the community with community tools? Are you <b>cheap</b> ... is getting stuff for free (as in beer) important to you?</p><br>";
   
   print "<table border='0'>";
   makeRankInput("Freedom crusading");
   makeRankInput("Tinkering");
   makeRankInput("Community");
   makeRankInput("Underdog");
   makeRankInput("Cheapness");
   print "</table><br>";
   
   print "<input type='submit' name='submit' value='Submit'><br>";
   
   print "<input type='checkbox' name='playing'> <font size=-1>I am just playing / please don't record my <a href='stats.php'>statistics</a>.</font><br>";
   
   print "</form></center>";
}

?>
